/****************************************************************
	MotorPWM.c 
	
	Motor control functions for PWM input motor drivers
	Contorol board CM01-A01  

							2015.7.14 M.Utsumi@ARCDEVICE

*****************************************************************/

#include"MotorPWM.h"
#include"Encoder.h"

// Pin assignment
// ch0
// P1 VCC----VCC
// P2 DIR----RE2
// P3 ENB----RE0	// brake is active low 
// P4 PWM----RE1	// pwm 1H
// P5 GND----GND

// ch1
// P1 VCC----VCC
// P4 DIR----RE4
// P5 ENB----RD0	// brake is active low 
// P3 PWM----RE3	// pwm 2H
// P5 GND----GND

// ch2
// P1 VCC----VCC
// P4 DIR----RD1
// P5 ENB----RD2	// brake is active low 
// P3 PWM----RE5	// pwm 3H
// P5 GND----GND


#define DIR0	_LATE2
#define ENB0	_LATE0	// L6203:ENABLE  iMDs03:BRAKE
#define DIR1	_LATE4
#define ENB1	_LATD0	// L6203:ENABLE  iMDs03:BRAKE
#define DIR2	_LATD1
#define ENB2	_LATD2	// L6203:ENABLE  iMDs03:BRAKE


#define TRIS_DIR0		TRISEbits.TRISE2
#define TRIS_ENB0		TRISEbits.TRISE0	
#define TRIS_DIR1		TRISEbits.TRISE4
#define TRIS_ENB1		TRISDbits.TRISD0	
#define TRIS_DIR2		TRISDbits.TRISD1
#define TRIS_ENB2		TRISDbits.TRISD2	


void MOTOR_Open( void )
{
	TRIS_DIR0 = 0;
	TRIS_ENB0 = 0;
	DIR0 = 1;	// CW
	ENB0 = 1;	// L6203:ENABLE=ON  iMDs03:BRAKE=OFF
 
	TRIS_DIR1 = 0;
	TRIS_ENB1 = 0;
	DIR1 = 1;	// CW
	ENB1 = 1;	// L6203:ENABLE=ON  iMDs03:BRAKE=OFF

	TRIS_DIR2 = 0;
	TRIS_ENB2 = 0;
	DIR2 = 1;	// CW
	ENB2 = 1;	// L6203:ENABLE=ON  iMDs03:BRAKE=OFF

	PTCON = 0x2000;	// 0010 0000 0000 0000
	/*
		15:   PTEN 	 = 0 PWM time base is off
		14:   N/A
		13:   PTSIDL = 1 PWM time base halts in CPU Idle mode
		12-8: N/A
		7-4: PTOPS	 = 000 1:1 postscale
		3-2: PTCKPS	 = 00 PWM time base input clock period is TCY (1:1 prescale)
		1-0: PTMOD	 = 0 PWM time base operates in Free Running mode
	*/

	PTMR	= 0;	// counter clear
	PTPER = 511;	// Time base period
	SEVTCMP = 0x0000;	
	/*
		15:  SEVTDIR = 0 Special Event Trigger will occur when the PWM time base is counting up
		14-0:SEVTCMP = 0 Special Event Compare Value bits
	*/
	PWMCON1 = 0x0F70; // 0000 1111 0111 0000
	/*
		15-12: N/A
		11:    PMOD4 = 1 PWM I/O pin pair is in Independent Output mode
		10:    PMOD3 = 1 PWM I/O pin pair is in Independent Output mode
		9:     PMOD2 = 1 PWM I/O pin pair is in Independent Output mode
		8:     PMOD1 = 1 PWM I/O pin pair is in Independent Output mode
		7:     PEN4H = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		6:     PEN3H = 1 PWMxHy pin eabled;
		5:     PEN2H = 1 PWMxHy pin eabled;
		4:     PEN1H = 1 PWMxHy pin eabled;
		2:     PEN4L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		2:     PEN3L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		1:     PEN2L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
		0:     PEN1L = 0 PWMxHy pin disabled; I/O pin becomes general purpose I/O
	*/
	PWMCON2bits.SEVOPS	= 0; // 1:1 postscale
	PWMCON2bits.IUE		= 0; // Updates to the active PxDC registers are synchronized to the PWM time base
	PWMCON2bits.OSYNC	= 0; // Output overrides via the PxOVDCON register occur on next TCY boundary
	PWMCON2bits.UDIS	= 0; // Updates from du
	PDC1 = 0;	// PWM Duty Cycle
	PDC2 = 0;	// PWM Duty Cycle
	PDC3 = 0;	// PWM Duty Cycle

	PTCONbits.PTEN 	= 1;	// PWM time base is on
	
}

void MOTOR_Close( void )
{
	PTCONbits.PTEN 	= 0;	// PWM time base is off
}


void MOTOR_Drive( WORD wCh, short sDuty )
{
	int dir, enb;
	short sDuty_abs;	 

	if( ENCODER_SimMode ){
 		ENCODER_SimCount(sDuty);
		return;
	}
	
	if( sDuty > MOTOR_MAX_VALUE ){	// free or brake
		dir = 0;
		enb = 0;	//  	
		sDuty_abs = 0;
		sDuty = 0;
	}else if( sDuty==0 ){		// STOP
		dir = 0;
		enb = 1;	//
		sDuty_abs = 0;
		sDuty = 0;
	}else if( sDuty>0 ){		// CW
		dir = 1;	//
		enb = 1;
		sDuty_abs =  sDuty;
	}else{					// CCW
		dir = 0;	//
		enb = 1;
		sDuty_abs = -sDuty;
	}

	switch( wCh ){
	case 0:
		DIR0 = dir;
		ENB0 = enb;
		PDC1 = sDuty_abs;
		break;
	case 1:
		DIR1 = dir;
		ENB1 = enb;
		PDC2 = sDuty_abs;
		break;
	case 2:
		DIR2 = dir;
		ENB2 = enb;
		PDC3 = sDuty_abs;
		break;
	}
}

